@@ -160,6 +160,28 @@
         activate_tabs();
     };
 
+    /**
+     * Avoids double-submit issues in the change_form.
+     */
+    $.fn.suit_form_debounce = function () {
+        var $form = $(this),
+            $saveButtons = $form.find('.submit-row button'),
+            submitting = false;
+
+        $form.submit(function () {
+            if (submitting) {
+                return false;
+            }
+
+            submitting = true;
+            $saveButtons.addClass('disabled');
+
+            setTimeout(function () {
+                $saveButtons.removeClass('disabled');
+                submitting = false;
+            }, 5000);
+        });
+    };
 
     $(function () {
 
